SystemOrganization addCategory: #Sitemap!

PRVisitor subclass: #Sitemap
	instanceVariableNames: 'serverUrl document root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sitemap'!

!Sitemap commentStamp: '<historical>' prior: 0!
http://www.google.com/schemas/sitemap/0.84/
https://www.google.com/webmasters/sitemaps/docs/en/protocol.html
http://www.xml.com/pub/a/2005/10/26/google-site-maps.html

Notes:
- a structure should know its url
- a structure should know its last modification!

!Sitemap class methodsFor: 'instance creation' stamp: 'pmm 12/9/2005 00:14'!
exmaple1
	| sitemap |
	sitemap := self forKernel: 'default' server: 'http://www.pier.com/'.
	! !

!Sitemap class methodsFor: 'instance creation' stamp: 'pmm 12/9/2005 00:19'!
forKernel: aKernelString server: aServerString
	| kernel root context sitemap |
	kernel := PRKernel instances detect: [ :each | each name = aKernelString ].
	root := kernel root.
	context := PRContext kernel: kernel structure: root.
	sitemap := self new serverUrl: aServerString; yourself.
	PRCurrentContext
		use: context
		during: [ sitemap visit: root ].
	^sitemap
	! !

!Sitemap methodsFor: 'private' stamp: 'pmm 12/9/2005 00:05'!
add: aPage
	| url location timestamp lastModification |
	url := XMLElement named: 'url'.
	location := XMLElement named: 'location'.
	location addContent: (XMLStringNode string:
		(String streamContents: [ :stream |
			stream nextPutAll: self serverUrl.
			aPage parents
				do: [ :each | stream nextPutAll: each title ]
				separatedBy: [ stream nextPut: '/' ] ]) ).
	url addElement: location.
	timestamp := self timestampOf: aPage.
	timestamp isNil ifFalse: [
		lastModification := XMLElement named: 'lastmod'.
		lastModification addContent: (XMLStringNode string: timestamp date yyyymmdd).
		url addElement: lastModification ].
	self root addElement: url! !

!Sitemap methodsFor: 'accessing' stamp: 'pmm 12/8/2005 23:57'!
context
	^PRCurrentContext value! !

!Sitemap methodsFor: 'accessing' stamp: 'pmm 12/8/2005 23:55'!
document
	^document! !

!Sitemap methodsFor: 'accessing' stamp: 'pmm 12/8/2005 23:55'!
document: anXmlDocument 
	document := anXmlDocument! !

!Sitemap methodsFor: 'testing' stamp: 'pmm 12/8/2005 23:47'!
ignoreChildrenOf: aPage
	^self ignore: aPage! !

!Sitemap methodsFor: 'testing' stamp: 'pmm 12/8/2005 23:43'!
ignore: aPage
	^aPage title = 'Environment'! !

!Sitemap methodsFor: 'initialize-release' stamp: 'pmm 12/8/2005 23:57'!
initialize
	super initialize.
	self document: (XMLDocument new
		version: '1.0';
		encoding: 'UTF-8';
		yourself).
	self root: (XMLElement named: 'urlset').
	self root namespace: nil uri: 'http://www.google.com/schemas/sitemap/0.84'.
	document addElement: root
	! !

!Sitemap methodsFor: 'private' stamp: 'pmm 12/8/2005 23:58'!
persistency
	^self context kernel persistency! !

!Sitemap methodsFor: 'accessing' stamp: 'pmm 12/8/2005 23:55'!
root
	^root! !

!Sitemap methodsFor: 'accessing' stamp: 'pmm 12/8/2005 23:56'!
root: anXmlElement 
	root := anXmlElement! !

!Sitemap methodsFor: 'accessing' stamp: 'pmm 12/8/2005 23:55'!
serverUrl
	^serverUrl! !

!Sitemap methodsFor: 'accessing' stamp: 'pmm 12/8/2005 23:56'!
serverUrl: aString 
	serverUrl := aString! !

!Sitemap methodsFor: 'private' stamp: 'pmm 12/9/2005 00:02'!
timestampOf: aStructure
	self persistency for: aStructure do: [ :each |
		^each timestamp ].
	^nil! !

!Sitemap methodsFor: 'visiting-structure' stamp: 'pmm 12/8/2005 23:47'!
visitPage: aPage
	(self ignore: aPage) ifFalse: [
		self add: aPage ].
	(self ignoreChildrenOf: aPage) ifFalse: [
		super visitPage: aPage ]! !

!Sitemap methodsFor: 'accessing' stamp: 'pmm 12/9/2005 00:22'!
xmlString
	^String streamContents: [ :stream |
		self document printCanonicalOn: stream ].! !

!Sitemap methodsFor: 'accessing' stamp: 'pmm 12/9/2005 00:25'!
xmlStringGZipCompressed
	| stream |
	stream := WriteStream on: (String new: 100).
	self document printCanonicalOn: stream.
	^stream contents! !
